//-------------------------------------------------
// TubeMeshGeometry.cs (c) 2006 by Charles Petzold
//-------------------------------------------------
using System;
using System.Windows.Media.Media3D;

namespace Petzold.RotatingRings
{
    public class TubeMeshGeometry
    {
        // Private fields.
        double radius = 1;
        double thickness = 0.1;
        double length = 1;
        int divisions = 10;

        // Public properties.
        public double Radius
        {
            set
            {
                if (value <= 0)
                    throw new Exception("TubeMeshGeometry.Radius must be " + 
                                        "greater than 0");
                radius = value;
            }
            get
            {
                return radius;
            }
        }
        public double Thickness
        {
            set
            {
                if (value <= 0)
                    throw new Exception("TubeMeshGeometry.Thickness must be " +
                                        "greater than 0");
                thickness = value;
            }
            get
            {
                return thickness;
            }
        }
        public double Length
        {
            set
            {
                if (value <= 0)
                    throw new Exception("TubeMeshGeometry.Length must be " +
                                        "greater than 0");
                length = value;
            }
            get
            {
                return length;
            }
        }
        public int Divisions
        {
            set
            {
                if (value <= 2)
                    throw new Exception("TubeMeshGeometry.Divisions must be " +
                                        "greater than 2");
                divisions = value;
            }
            get
            {
                return divisions;
            }
        }
        public MeshGeometry3D MeshGeometry
        {
            get
            {
                MeshGeometry3D meshgeo = new MeshGeometry3D();

                // First calculate the Positions collection.
                double angle, x, y, z = Length / 2;

                // Calculate inner circle, positive z end.
                // Positions indices start at 0.
                for (int i = 0; i < Divisions; i++)
                {
                    angle = 2 * Math.PI * i / Divisions;
                    x = Radius * Math.Cos(angle);
                    y = Radius * Math.Sin(angle);
                    meshgeo.Positions.Add(new Point3D(x, y, z));
                }

                // Calculate outer circle, positive z end.
                // Positions indices start at Divisions.
                for (int i = 0; i < Divisions; i++)
                {
                    angle = 2 * Math.PI * i / Divisions;
                    x = (Radius + Thickness) * Math.Cos(angle);
                    y = (Radius + Thickness) * Math.Sin(angle);
                    meshgeo.Positions.Add(new Point3D(x, y, z));
                }

                z *= -1;

                // Calculate inner circle, negative z end.
                // Positions indices start at 2 * Divisions.
                for (int i = 0; i < Divisions; i++)
                {
                    angle = 2 * Math.PI * i / Divisions;
                    x = Radius * Math.Cos(angle);
                    y = Radius * Math.Sin(angle);
                    meshgeo.Positions.Add(new Point3D(x, y, z));
                }

                // Calculate outer circle, negative z end.
                // Positions indices start at 3 * Divisions.
                for (int i = 0; i < Divisions; i++)
                {
                    angle = 2 * Math.PI * i / Divisions;
                    x = (Radius + Thickness) * Math.Cos(angle);
                    y = (Radius + Thickness) * Math.Sin(angle);
                    meshgeo.Positions.Add(new Point3D(x, y, z));
                }

                // Next take care of TriangleIndices.
                for (int i = 0; i < Divisions; i++)
                {
                    int iNext = (i + 1) % Divisions;

                    // Edge on positive z end.
                    meshgeo.TriangleIndices.Add(i);
                    meshgeo.TriangleIndices.Add(Divisions + i);
                    meshgeo.TriangleIndices.Add(iNext);

                    meshgeo.TriangleIndices.Add(Divisions + i);
                    meshgeo.TriangleIndices.Add(Divisions + iNext);
                    meshgeo.TriangleIndices.Add(iNext);

                    // Inside.
                    meshgeo.TriangleIndices.Add(i);
                    meshgeo.TriangleIndices.Add(iNext);
                    meshgeo.TriangleIndices.Add(2 * Divisions + i);

                    meshgeo.TriangleIndices.Add(2 * Divisions + i);
                    meshgeo.TriangleIndices.Add(iNext);
                    meshgeo.TriangleIndices.Add(2 * Divisions + iNext);
 
                    // Edge on negative z end.
                    meshgeo.TriangleIndices.Add(3 * Divisions + i);
                    meshgeo.TriangleIndices.Add(2 * Divisions + i);
                    meshgeo.TriangleIndices.Add(3 * Divisions + iNext);

                    meshgeo.TriangleIndices.Add(2 * Divisions + i);
                    meshgeo.TriangleIndices.Add(2 * Divisions + iNext);
                    meshgeo.TriangleIndices.Add(3 * Divisions + iNext);

                    // Outside.
                    meshgeo.TriangleIndices.Add(3 * Divisions + i);
                    meshgeo.TriangleIndices.Add(3 * Divisions + iNext);
                    meshgeo.TriangleIndices.Add(Divisions + i);

                    meshgeo.TriangleIndices.Add(Divisions + i);
                    meshgeo.TriangleIndices.Add(3 * Divisions + iNext);
                    meshgeo.TriangleIndices.Add(Divisions + iNext);
                }
                return meshgeo;
            }
        }
    }
}
